@@ -26,23 +26,26 @@ import xdis.magics as magics
 from xdis.load import check_object_path
 from xdis import IS_PYPY
 
-#----- configure this for your needs
+# ----- configure this for your needs
 
-python_versions = [v for v in magics.python_versions if
-                       re.match('^[0-9.]+$', v)]
+python_versions = [v for v in magics.python_versions if re.match("^[0-9.]+$", v)]
 
-python_versions.append('1.4')
-
-# Until there is a final release and magics is updated...
-python_versions.append('3.7.0')
+python_versions.append("1.4")
 
 TEST_VERSIONS = tuple(python_versions)
 
 
-PYPY_TEST_VERSIONS=(('pypy-2.6.1', '2.7'), ('pypy-5.0.1', '2.7'),
-                    ('pypy3-2.4.0', '3.2'))
+PYPY_TEST_VERSIONS = (
+    ("pypy-2.6.1", "2.7"),
+    ("pypy-5.0.1", "2.7"),
+    ("pypy3-2.4.0", "3.2"),
+    ("pypy3.5-6.0.0", "3.5"),
+    ("pypy3.5-5.9.0", "3.5"),
+    ("pypy3.6-7.1.0", "3.6"),
+    ("pypy3.6-7.1.1", "3.6"),
+)
 
-#-----
+# -----
 
 
 import os, time, shutil, sys
@@ -51,57 +54,69 @@ from fnmatch import fnmatch
 from xdis import main, PYTHON3, PYTHON_VERSION
 from xdis.verify import verify_file
 
-LONG_PYTHON_VERSION = ("%s.%s.%s" %
-                       (sys.version_info[0],
-                        sys.version_info[1],
-                        sys.version_info[2]))
+LONG_PYTHON_VERSION = "%s.%s.%s" % (
+    sys.version_info[0],
+    sys.version_info[1],
+    sys.version_info[2],
+)
 
-target_base = '/tmp/py-dis/'
-if hasattr(os.path.join, '__code__'):
+target_base = "/tmp/py-dis/"
+if hasattr(os.path.join, "__code__"):
     lib_prefix = os.path.dirname(os.path.join.__code__.co_filename)
 else:
     lib_prefix = os.path.dirname(os.path.join.func_code.co_filename)
 
-PY = ('*.py', )
-PYC = ('*.pyc', )
-PYO = ('*.pyo', )
-PYOC = ('*.pyc', '*.pyo')
+PY = ("*.py",)
+PYC = ("*.pyc",)
+PYO = ("*.pyo",)
+PYOC = ("*.pyc", "*.pyo")
 
 my_dir = os.path.dirname(__file__)
-test_options = {
-    'simple': (os.path.join(my_dir, 'simple_source'),
-               PY, 'simple-source')
-    }
+test_options = {"simple": (os.path.join(my_dir, "simple_source"), PY, "simple-source")}
 
 for vers in TEST_VERSIONS:
     short_vers = vers[:3]
-    test_options[vers] = (lib_prefix, PYC, 'python-lib'+short_vers)
+    test_options[vers] = (lib_prefix, PYC, "python-lib" + short_vers)
 
-test_options['1.4'] = (lib_prefix, PYC, 'python-lib1.4')
+test_options["1.4"] = (lib_prefix, PYC, "python-lib1.4")
 
 for vers, short_vers in PYPY_TEST_VERSIONS:
-    test_options[vers] = (os.path.join(lib_prefix, vers, 'lib-python', short_vers),
-                          PYC, 'python'+short_vers)
+    test_options[vers] = (
+        os.path.join(lib_prefix, vers, "lib-python", short_vers),
+        PYC,
+        "python" + short_vers,
+    )
 
 
-
-def do_tests(src_dir, patterns, target_dir, start_with=None,
-                 do_verify=False, max_files=800, do_compile=False,
-                 verbose=False):
-
+def do_tests(
+    src_dir,
+    patterns,
+    target_dir,
+    start_with=None,
+    do_verify=False,
+    max_files=800,
+    do_compile=False,
+    verbose=False,
+):
     def visitor(files, dirname, names):
         files.extend(
-            [os.path.normpath(os.path.join(dirname, n))
-                 for n in names
-                    for pat in patterns
-                        if fnmatch(n, pat)])
+            [
+                os.path.normpath(os.path.join(dirname, n))
+                for n in names
+                for pat in patterns
+                if fnmatch(n, pat)
+            ]
+        )
 
     def file_matches(files, root, basenames, patterns):
         files.extend(
-            [os.path.normpath(os.path.join(root, n))
-                 for n in basenames
-                    for pat in patterns
-                        if fnmatch(n, pat)])
+            [
+                os.path.normpath(os.path.join(root, n))
+                for n in basenames
+                for pat in patterns
+                if fnmatch(n, pat)
+            ]
+        )
 
     files = []
     if do_compile:
@@ -119,10 +134,13 @@ def do_tests(src_dir, patterns, target_dir, start_with=None,
     if PYTHON3:
         for root, dirname, names in os.walk(os.curdir):
             files.extend(
-                [os.path.normpath(os.path.join(root, n))
-                     for n in names
-                        for pat in patterns
-                            if fnmatch(n, pat)])
+                [
+                    os.path.normpath(os.path.join(root, n))
+                    for n in names
+                    for pat in patterns
+                    if fnmatch(n, pat)
+                ]
+            )
             pass
         pass
     else:
@@ -133,13 +151,13 @@ def do_tests(src_dir, patterns, target_dir, start_with=None,
         try:
             start_with = files.index(start_with)
             files = files[start_with:]
-            print('>>> starting with file', files[0])
+            print(">>> starting with file", files[0])
         except ValueError:
             pass
 
     if len(files) > max_files:
-        files = [file for file in files if not 'site-packages' in file]
-        files = [file for file in files if not 'test' in file]
+        files = [file for file in files if not "site-packages" in file]
+        files = [file for file in files if not "test" in file]
         if len(files) > max_files:
             files = files[:max_files]
             pass
@@ -148,19 +166,20 @@ def do_tests(src_dir, patterns, target_dir, start_with=None,
         os.chdir(cwd)
         return
 
-    output = open(os.devnull,"w")
+    output = open(os.devnull, "w")
     # output = sys.stdout
     start_time = time.time()
     print(time.ctime())
     for i, bc_file in enumerate(files):
         if verbose:
             print(os.path.join(src_dir, bc_file))
-        if sys.version_info >= (3, 4, 0) and bc_file.endswith('.py'):
+        if sys.version_info >= (3, 4, 0) and bc_file.endswith(".py"):
             check_bc_file = check_object_path(bc_file)
             if not osp.exists(check_bc_file):
                 basename = osp.basename(bc_file)[0:-3]
-                new_bc_file = tempfile.mkstemp(prefix=basename + '-',
-                                           suffix='.pyc', text=False)[1]
+                new_bc_file = tempfile.mkstemp(
+                    prefix=basename + "-", suffix=".pyc", text=False
+                )[1]
                 py_compile.compile(bc_file, cfile=new_bc_file, doraise=True)
                 bc_file = new_bc_file
 
@@ -170,13 +189,14 @@ def do_tests(src_dir, patterns, target_dir, start_with=None,
             verify_file(file, bc_filename)
         if i % 100 == 0 and i > 0:
             print("Processed %d files" % (i))
-    print("Processed %d files, total" % (i+1))
+    print("Processed %d files, total" % (i + 1))
     print(time.ctime())
     elapsed_time = time.time() - start_time
     print("%g seconds" % elapsed_time)
     os.chdir(cwd)
 
-if __name__ == '__main__':
+
+if __name__ == "__main__":
     import getopt
 
     do_verify = False
@@ -187,24 +207,23 @@ if __name__ == '__main__':
     test_version = None
 
     if len(sys.argv) == 1:
-        sys.argv[1] == '--simple'
+        sys.argv[1] == "--simple"
 
     test_options_keys = list(test_options.keys())
     test_options_keys.sort()
-    opts, args = getopt.getopt(sys.argv[1:], '',
-                               ['start-with=',
-                                'max-files=',
-                                'compile',
-                                'verify', ] \
-                               + test_options_keys )
+    opts, args = getopt.getopt(
+        sys.argv[1:],
+        "",
+        ["start-with=", "max-files=", "compile", "verify"] + test_options_keys,
+    )
     for opt, val in opts:
-        if opt == '--verify':
+        if opt == "--verify":
             do_verify = True
-        elif opt == '--compile':
+        elif opt == "--compile":
             do_compile = True
-        elif opt == '--start-with':
+        elif opt == "--start-with":
             start_with = val
-        elif opt == '--max-files':
+        elif opt == "--max-files":
             max_files = int(val)
         elif opt[2:] in test_options_keys:
             test_version = opt[2:]
@@ -212,35 +231,38 @@ if __name__ == '__main__':
             pass
         pass
 
-    if test_version == 'simple':
+    if test_version == "simple":
         if PYTHON_VERSION > 2.6:
-            test_dirs.append((os.path.join(my_dir, 'simple_2.7'),
-                             PY, 'simple-source'))
+            test_dirs.append((os.path.join(my_dir, "simple_2.7"), PY, "simple-source"))
             pass
         pass
     if do_verify:
         if test_version is None:
             test_dirs.append(test_options[LONG_PYTHON_VERSION])
-        elif test_version != 'simple':
+        elif test_version != "simple":
             if LONG_PYTHON_VERSION != test_version:
-                sys.stderr.write("When --verify is used, the running version %s\n"
-                                 "has to be the same as the version you want to test %s\n"
-                                 % (LONG_PYTHON_VERSION, test_version))
+                sys.stderr.write(
+                    "When --verify is used, the running version %s\n"
+                    "has to be the same as the version you want to test %s\n"
+                    % (LONG_PYTHON_VERSION, test_version)
+                )
                 sys.exit(1)
             pass
         pass
 
-    if PYTHON_VERSION >= 3.5 and IS_PYPY:
-        print("### Doesn't work on Python 3.5 or greater")
-        sys.exit(0)
-
     for src_dir, pattern, target_dir in test_dirs:
         if os.path.exists(src_dir):
             target_dir = os.path.join(target_base, target_dir)
             if os.path.exists(target_dir):
                 shutil.rmtree(target_dir, ignore_errors=1)
-            do_tests(src_dir, pattern, target_dir, start_with,
-                         do_verify=do_verify,
-                         max_files=max_files, do_compile=do_compile)
+            do_tests(
+                src_dir,
+                pattern,
+                target_dir,
+                start_with,
+                do_verify=do_verify,
+                max_files=max_files,
+                do_compile=do_compile,
+            )
         else:
             print("### Path %s doesn't exist; skipping" % src_dir)
