@@ -32,7 +32,7 @@ and the value is its canonic versions string, e.g. '2.7'
 PYTHON_MAGIC_INT: The magic integer for the current running Python interpreter
 """
 
-import imp, re, struct, sys
+import re, struct, sys
 from xdis import IS_PYPY
 
 IS_PYPY3 = (48, 112, 160)
@@ -88,7 +88,17 @@ by_magic = {}
 by_version = {}
 magicint2version = {}
 versions = {}
-PYTHON_MAGIC_INT = magic2int(imp.get_magic())
+
+if sys.version_info >= (3, 4):
+    import importlib
+
+    MAGIC = importlib.util.MAGIC_NUMBER
+    PYTHON_MAGIC_INT = magic2int(MAGIC)
+else:
+    import imp
+
+    MAGIC = imp.get_magic()
+    PYTHON_MAGIC_INT = magic2int(MAGIC)
 
 # The magic word is used to reject .pyc files generated by other
 # Python versions.  It should change for each incompatible change to
@@ -301,10 +311,11 @@ add_canonic_versions("2.6 2.6.6 2.6.7 2.6.8 2.6.9", "2.6a1")
 add_canonic_versions(
     "2.7.0 2.7.1 2.7.2 2.7.2 2.7.3 2.7.4 2.7.5 2.7.6 2.7.7 "
     "2.7.8 2.7.9 2.7.10 2.7.11 2.7.12 2.7.13 2.7.14 2.7.15 "
-    "2.7.16",
+    "2.7.15candidate1 "
+    "2.7.16 "
+    "2.7.17rc1 2.7.17candidate1",
     "2.7",
 )
-add_canonic_versions("2.7.15candidate1", "2.7")
 add_canonic_versions("3.0 3.0.0 3.0.1", "3.0a5")
 add_canonic_versions("3.1 3.1.0 3.1.1 3.1.2 3.1.3 3.1.4 3.1.5", "3.1a0+")
 add_canonic_versions("3.2 3.2.0 3.2.1 3.2.2 3.2.3 3.2.4 3.2.5 3.2.6", "3.2a2")
@@ -327,7 +338,7 @@ add_canonic_versions("3.5.3pypy", "3.5pypy")
 add_canonic_versions("3.6.1pypy", "3.6pypy")
 add_canonic_versions("2.7.8Pyston", "2.7.7Pyston")
 add_canonic_versions("3.7.0alpha3", "3.7.0alpha3")
-add_canonic_versions("3.7 3.7.0beta5 3.7.1 3.7.2 3.7.3 3.7.4", "3.7.0")
+add_canonic_versions("3.7 3.7.0beta5 3.7.1 3.7.2 3.7.3 3.7.4 3.7.5", "3.7.0")
 add_canonic_versions("3.8.0alpha0 3.8.0alpha3 3.8.0a0", "3.8.0a3+")
 add_canonic_versions("3.8.0 3.8 3.8.0candidate1", "3.8.0rc1+")
 
